"""py2ch User Agent.
"""
#
# Copyright (c) 2005 Satoshi Fukutomi <fuktommy@inter7.jp>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#

import os
import time
import socket
import urllib
import urlparse
from StringIO import StringIO

import config

__all__ = ["URL", "Monazilla", "geturl", "posturl",
           "HTTPError302", "HTTPError404", "HTTPError416"]

socket.setdefaulttimeout(config.timeout)

class HTTPError302(IOError): pass
class HTTPError404(IOError): pass
class HTTPError416(IOError): pass

urllib.URLopener.version = config.version
class Monazilla(urllib.FancyURLopener):
    def __init__(self, *args):
        self.version = "%s (%s)" % (config.agent, config.version)
        urllib.FancyURLopener.__init__(self, *args)
        self.proxies = {}

    def http_error_302(self, url, fp, errcode, errmsg, headers, data=None):
        void = fp.read()
        fp.close()
        raise HTTPError302, ('http error', errcode, errmsg, headers)

    def http_error_416(self, url, fp, errcode, errmsg, headers, data=None):
        void = fp.read()
        fp.close()
        raise HTTPError416, ('http error', errcode, errmsg, headers)


def gunzip(buf):
    from gzip import GzipFile
    sio = StringIO(buf)
    try:
        gz = GzipFile(fileobj=sio)
        buf = gz.read()
    finally:
        gz.close()
    return buf


class URL:
    """URL Object."""

    def __init__(self, urlstr):
        if urlstr.find("://") < 0:
            urlstr = "http://" + urlstr
        self.urlstr = urlstr
        buf = urlparse.urlparse(urlstr)
        self.filename = "%s/%s%s" % (config.cachedir, buf[1], buf[2])

        if os.path.exists(self.filename):
            self.mtime = os.path.getmtime(self.filename)
            self.size = os.path.getsize(self.filename)
        else:
            self.mtime = 0
            self.size = 0

    def __str__(self):
        return self.urlstr

    def _get(self, new=False, diff=False):
        """Get file from network."""
        urlopener = Monazilla()
        timefmt = "%a, %d %b %Y %H:%M:%S GMT"
        if new and self.mtime:
            mtime = time.strftime(timefmt, time.gmtime(self.mtime))
            urlopener.addheader("If-Modified-Since", mtime)
            urlopener.addheader("Accept-Encoding", "gzip")
        else:
            urlopener.addheader("Accept-Encoding", "gzip")
        f = urlopener.open(self.urlstr)
        buf = f.read()
        if f.info().getheader("Content-Encoding", "") == "gzip":
            buf = gunzip(buf)
        f.close()
        return buf

    def get(self, new=False, diff=False):
        """Get file from network."""
        mode = "wb"
        try:
            buf = self._get(new=new)
            if buf:
                self.write(mode, buf)
        except HTTPError404:
            pass
        except socket.error:
            pass

    def write(self, mode, buf):
        dirname = os.path.dirname(self.filename)
        if not os.path.exists(dirname):
            os.makedirs(dirname)
        f = file(self.filename, mode)
        f.write(buf)
        f.close()

    def open(self):
        try:
            return file(self.filename)
        except IOError:
            return StringIO("")

def geturl(urlstr):
    urlopener = Monazilla()
    urlopener.addheader("Accept-Encoding", "gzip")
    try:
        f = urlopener.open(urlstr)
    except IOError:
        return [""]
    buf = f.read()
    if f.info().getheader("Content-Encoding", "") == "gzip":
        buf = gunzip(buf)
    f.close()
    return buf.splitlines()

def posturl(urlstr, data, addheader=None):
    urlopener = Monazilla()
    if addheader:
        for pair in addheader:
            urlopener.addheader(*pair)
    try:
        f = urlopener.open(urlstr, data)
    except IOError:
        return [""]
    buf = f.read()
    headers = f.headers
    f.close()
    return headers, buf.splitlines()
